<?php
// (c) Copyright 2002-2015 by authors of the Tiki Wiki CMS Groupware Project
//
// All Rights Reserved. See copyright.txt for details and a complete list of authors.
// Licensed under the GNU LESSER GENERAL PUBLIC LICENSE. See license.txt for details.
// $Id: Plugin.php 56560 2015-10-24 15:57:40Z lindonb $

//this script may only be included - so its better to die if called directly.
if (strpos($_SERVER['SCRIPT_NAME'], basename(__FILE__)) !== false) {
	header('location: index.php');
	exit;
}

/**
 * Class Table_Plugin
 *
 * For use by plugins to apply tablesorter to tables generated by the plugin. Creates parameters that
 * can be merged in with the plugin's parameters and generates settings from user input that can be
 * used in Table_Factory
 */
class Table_Plugin
{
	/**
	 * Standard tablesorter parameters for a plugin
	 * @var array
	 */
	public $params = array();

	/**
	 * Holds the settings created by setSettings function below
	 * @var
	 */
	public $settings;

	/**
	 * Creates parameters that can be appended to a plugin's native parameters so the user can
	 * set tablesorter functionality
	 */
	public function createParams()
	{
		$this->params = array(
			'server' => array(
				'required' => false,
				'name' => tra('Server Side Processing'),
				'description' => tr(
					'Enter %0y%1 to have the server do the sorting and filtering through Ajax and %0n%1 to have the
					browser do it (n is the default). Set to %0y%1 (and also set the Paginate parameter
					(%0tspaginate%1)) if you do not want all rows fetched at once, but rather fetch rows as you
					paginate, filter or sort.', '<code>', '</code>'
				),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => 'n',
				'filter' => 'striptags',
			),
			'sortable' => array(
				'required' => false,
				'name' => tra('Overall Sort Settings'),
				'description' => tr(
					'Serves as the overall switch for turning jQuery Tablesorter on (also for filtering) as well as
					overall sort settings. Enter %0y%1 to allow sorting and %0n%1 to disallow (n is the default).
					Enter %0type:save%1 to allow sorts to be saved between page refreshes.
					Enter %0type:%2reset%3;text:*****%1 to allow sorting and show an unsort button with
					custom text. Enter %0type:%2savereset%3;text:buttontext%1 to allow the same for saved sorts.',
					'<code>', '</code>', '<strong>', '</strong>'
				),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => 'n',
				'filter' => 'striptags',
			),
			'sortList' => array(
				'required' => false,
				'name' => tra('Pre-sorted Columns'),
				'description' => tr(
					'Bracketed numbers for column number (first column = 0) and sort direction
					(%20%3 = ascending, %21%3 = descending, %2n%3 = no sort, %2y%3 = allow sorting but no pre-sort),
					for example: %0. If the first pre-sorted or no filter column is not the first column, then you
					should use the %2y%3 parameter (as in %1) to assign all previous columns.',
					'<code>~np~[0,y],[1,0],[2,n]~/np~</code>', '<code>~np~[0,y]~/np~</code>', '<code>', '</code>'
				),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
			'tsortcolumns' => array(
				'required' => false,
				'name' => tra('Sort Settings by Column'),
				'description' => tr(
					'Set %0type%1 and %0group%1 settings for each column, using %0|%1 to separate columns. To
					show group headings upon page load, the Pre-sorted Columns parameter (%0sortList%1) will need to be
					set for a column with a group setting. Group will not work in plugins where the Server Side
					Processing parameter (%0server%1) is set to %0y%1.', '<code>', '</code>')
				. '<br />' . tr('%0type%1 tells the sorter what type of date is being sorted and choices include:
					%0text%1, %0digit%1, %0currency%1, %0percent%1, %0usLongDate%1, %0shortDate%1, %0isoDate%1,
					%0dateFormat-ddmmyyyy%1, %0ipAddress%1, %0url%1, %0time%1.
					Also handle strings in numeric columns with %0string-min%1 and %0string-max%1. Handle empty cells
					with %0empty-top%1, %0empty-bottom%1 or %0empty-zero%1.', '<code>', '</code>')
				. '<br />' . tr('%0group%1 creates automatic row headings upon sort with the heading text determined by
					the setting as follows: %0letter%1 (first letter), %0word%1 (first word), %0number%1, %0date%1,
					%0date-year%1, %0date-month%1, %0date-day%1, %0date-week%1, %0date-time%1. %0letter%1 and %0word%1
					can be extended, e.g., %0word-2%1 shows first 2 words. %0number-10%1 will group rows in blocks of
					ten. Group will not work in plugins where the Server Side Processing parameter (%0server%1) is
					set to %0y%1.', '<code>', '</code>'
				),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
			'tsfilters' => array(
				'required' => false,
				'name' => tra('Column Filters'),
				'description' => tr(
					'Enter %0y%1 for a blank text filter on all columns, or %0n%1 for no filters. Or set custom column
					filters separated by %0|%1 for each column for the following filter choices and parameters:',
					'<code>', '</code>'
				)
					. '<br> <b>Text - </b><code>type:text;placeholder:xxxx</code><br>' .
					tra('(For PluginTrackerlist this will be an exact search, for other plugins partial values will work.)') . '<br>
					<b>Dropdown - </b><code>type:dropdown;placeholder:****;option:****;option:****;option:****</code> <br>' .
					tr('Options generated automatically if not set and the %0server%1 parameter is not %0y%1.', '<code>', '</code>') . '<br>' .
					tr('Use %0value=Display label%1 to have the option value be different than the displayed label in
					the dropdown.', '<code>', '</code>') . '<br>
					<b>' . tra('Date range - ') . '</b><code>type:date;format:yy-mm-dd;from:2013-06-30;to:2020-12-31</code><br>' .
					tra('(from and to values set defaults for these fields when user clicks on the input field)') . '<br>
					<b>' . tra('Numeric range - ') . '</b><code>type:range;from:0;to:50</code><br>
					<b>' . tra('No filter - ') . '</b><code>type:nofilter</code><br>' .
					tr(
						'For example: %0tsfilters="type:dropdown;placeholder:Type to filter..."%1 would result in a dropdown
						filter on the first column with all unique values in that column in the dropdown list.'
						, '<code>', '</code>'),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
			'tsfilteroptions' => array(
				'required' => false,
				'name' => tra('Filter Options'),
				'description' => tr(
					'The following options are available: %0reset%1 (adds button to take off filters), and %0hide%1
					(Filters are revealed upon mouseover. Hide doesn\'t work when date and range filters are used.).
					To use both, set %0tsfilteroptions="type:reset;text:button text;style:hide"%1', '<code>', '</code>'
				),
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
			'tspaginate' => array(
				'required' => false,
				'name' => tra('Paginate'),
				'description' => tr(
					'Enter %0y%1 to set default values based on the site setting for maximum records in listings (on the
				 	pagination table of the Look & Feel admin panel). Set to %0n%1 (and %0server%1 cannot be set to
				 	%0y%1) for no pagination. Set custom values as in the following example: ',
						'<code>', '</code>') .
					'<code>max:40;expand:60;expand:100;expand:140</code>',
				'since' => '12.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
			'tscolselect' => array(
				'required' => false,
				'name' => tra('Column Select'),
				'description' => tr(
					'Add a button for hiding and re-showing columns. Also sets priority for dropping columns when
				 	browser is too narrow. Set each column to a number between 1 and 6 (1 is highest priority and last
				 	to be dropped) or to %0critical%1 to never hide or drop. An example with 4 columns:',
						'<code>', '</code>') .
					'<code>tscolselect="critical|4|5|6"</code>',
				'since' => '14.0',
				'doctype' => 'tablesorter',
				'default' => '',
				'filter' => 'striptags',
				'advanced' => true,
			),
		);
	}

	/**
	 * To be used within plugin program to convert user parameter settings into the settings array
	 * that can be used by Table_Factory to generate the necessary jQuery
	 *
	 * @param null   $id				//html element id for table and surrounding div
	 * @param string $sortable			//see params above
	 * @param null   $sortList			//see params above
	 * @param string $tsortcolumns		//see params above
	 * @param null   $tsfilters			//see params above
	 * @param null   $tsfilteroptions	//see params above
	 * @param null   $tspaginate		//see params above
	 * @param null   $ajaxurl			//only needed if ajax will be used to pull partial record sets
	 * @param null   $totalrows			//only needed if ajax will be used to pull partial record sets
	 */
	public function setSettings ($id = null, $server = 'n', $sortable = 'n', $sortList = null, $tsortcolumns = null,
		$tsfilters = null, $tsfilteroptions = null, $tspaginate = null, $tscolselect = null, $ajaxurl = null,
		$totalrows = null)
	{
		$s = array();

		//id
		if (!empty($id)) {
			$s['id'] = $id;
		}

		//sortable
		switch ($sortable) {
			case 'y':
			case 'server':
				$s['sorts']['type'] = true;
				break;
			case 'n':
				$s['sorts']['type'] = false;
				break;
			default:
				$sp = $this->parseParam($sortable);
				if (isset($sp[0]['type'])) {
					$s['sorts']['type'] = $sp[0]['type'];
				}
		}

		//sortlist
		if (!empty($sortList) && (!isset($s['sorts']['type']) || $s['sorts']['type'] !== false)) {
			$crop = substr($sortList, 1);
			$crop = substr($crop, 0, -1);
			$slarray = explode('],[', $crop);
			if (is_array($slarray)) {
				foreach ($slarray as $l) {
					$lpieces = explode(',', $l);
					if (isset($lpieces[1])) {
						switch ($lpieces[1]) {
							case '0':
								$dir = 'asc';
								break;
							case '1':
								$dir = 'desc';
								break;
							case 'y':
								$dir = true;
								break;
							case 'n':
								$dir = false;
								break;
							default:
								if($s['sorts']['type'] !== false) {
									$dir = true;
								} else {
									$dir = false;
								}
						}
						if ($dir === false || $dir === true) {
							$s['columns'][$lpieces[0]]['sort']['type'] = $dir;
						} else {
							$s['columns'][$lpieces[0]]['sort']['dir'] = $dir;
						}
					}
				}
			}
		}

		if (!empty($tsortcolumns)) {
			$tsc = $this->parseParam($tsortcolumns);
			if (is_array($tsc)) {
				foreach ($tsc as $col => $sortinfo) {
					if (isset($s['columns'][$col]['sort'])) {
						$s['columns'][$col]['sort'] = $s['columns'][$col]['sort'] + $sortinfo;
					} else {
						$s['columns'][$col]['sort'] = $sortinfo;
					}
				}
				ksort($s['columns']);
			}
		} else {
			$s['sorts']['group'] = false;
		}

		//tsfilters
		if (!empty($tsfilters)) {
			switch ($tsfilters) {
				case 'y':
					$s['filters']['type'] = 'text';
					break;
				case 'n':
					$s['filters']['type'] = false;
					break;
				default:
					$tsf = $this->parseParam($tsfilters);
					if (is_array($tsf)) {
						foreach ($tsf as $col => $filterinfo) {
							if (isset($filterinfo) && $filterinfo['type'] === 'dropdown'
								&& !empty($filterinfo['options'])) {
								foreach ($filterinfo['options'] as $key => $value) {
									$filterinfo['options'][$key] = str_replace('=', '|', $value);
								}
							}
							if (isset($s['columns'][$col]['filter'])) {
								$s['columns'][$col]['filter'] = $s['columns'][$col]['filter'] + $filterinfo;
							} else {
								$s['columns'][$col]['filter'] = $filterinfo;
							}
						}
					}
			}
		}

		//tsfilteroptions
		if (!empty($tsfilteroptions) && !empty($s['filters']['type'])) {
			$tsfo = $this->parseParam($tsfilteroptions);
			switch ($tsfo[0]['type']) {
				case 'reset':
					$s['filters']['type'] = 'reset';
					break;
				case 'hide':
					$s['filters']['hide'] = true;
					break;
			}
		}

		//tspaginate
		if (empty($tspaginate)) {
			$tspaginate = $server === 'y' ? 'y' : '';
		}
		if (!empty($tspaginate)) {
			$tsp = $this->parseParam($tspaginate);
			//pagination must be on if server side processing is on ($server == 'y')
			if (is_array($tsp[0]) || $tsp[0] !== 'n' || ($tsp[0] === 'n' && $server === 'y')) {
				if (is_array($tsp[0])) {
					$s['pager'] = $tsp[0];
					if (isset($s['pager']['expand']) && is_array($s['pager']['expand'])) {
						if (isset($s['pager']['max']) && $s['pager']['max'] > 0) {
							$s['pager']['expand'] = array_merge(array($s['pager']['max']), $s['pager']['expand']);
						} else {
							$s['pager']['max'] = min($s['pager']['expand']);
						}
						$s['pager']['expand'] = array_unique($s['pager']['expand']);
						sort($s['pager']['expand']);
					}
				}
				$s['pager']['type'] = true;
			} elseif ($tsp[0] === 'n' && $server === 'n') {
				$s['pager']['type'] = false;
			}
		}

		//tscolselect
		if (!empty($tscolselect)) {
			$tscs = $this->parseParam($tscolselect);
			if (is_array($tscs)) {
				$s['colselect']['type'] = true;
				foreach ($tscs as $col => $priority) {
					$s['columns'][$col]['priority'] = $priority;
				}
			}
		}

		//ajaxurl
		if (!empty($ajaxurl) && $server === 'y') {
			$url = $this->getAjaxurl($ajaxurl);
			$s['ajax']['url']['file'] = $url['path'];
			$s['ajax']['url']['query'] = $url['query'];
			$s['ajax']['type'] = true;
		} else {
			$s['ajax']['type'] = false;
		}

		//totalrows
		if (!empty($totalrows)) {
			$s['total'] = $totalrows;
		}

		$this->settings = $s;

	}

	/**
	 * Utility to convert string entered by user for a parameter setting to an array
	 *
	 * @param $param
	 *
	 * @return array
	 */
	function parseParam ($param)
	{
		if (!empty($param)) {
			$ret = explode('|', $param);
			foreach ($ret as $key => $pipe) {
				$ret[$key] = strpos($pipe, ';') !== false ? explode(';', $pipe) : $pipe;
				if (!is_array($ret[$key])) {
					if (strpos($ret[$key], ':') !== false) {
						$colon = explode(':', $ret[$key]);
						unset($ret[$key]);
						if ($colon[1] == 'nofilter') {
							$colon[1] = false;
						}
						$ret[$key][$colon[0]] = $colon[1];
					}
				} elseif (is_array($ret[$key])) {
					foreach ($ret[$key] as $key2 => $subparam) {
						if (strpos($subparam, ':') !== false) {
							$colon = explode(':', $subparam);
							unset($ret[$key][$key2]);
							if ($colon[0] == 'expand' || $colon[0] == 'option') {
								if ($colon[0] == 'option') {
									$colon[0] = 'options';
								}
								$ret[$key][$colon[0]][] = $colon[1];
							} else {
								$ret[$key][$colon[0]] = $colon[1];
							}
						}
					}
				}
			}
			ksort($ret);
			return $ret;
		} else {
			return $param;
		}
	}

	/**
	 * Utility to add ajax parameters to URL
	 *
	 * @param $ajaxurl
	 *
	 * @return string
	 */
	private function getAjaxurl($ajaxurl)
	{
		$str = '{sort:sort}&{filter:filter}';
		$url = parse_url($ajaxurl);
		if (isset($url['query'])) {
			$url['query'] = '?' .  $url['query'] . '&' . $str;
		} else {
			$url['query'] = '?' . $str;
		}
		return $url;
	}

}
